@@ -25,6 +25,7 @@
 #include <QApplication>
 #include <QClipboard>
 #include <QGraphicsGridLayout>
+#include <QGraphicsLinearLayout>
 #include <QGraphicsProxyWidget>
 #include <QFontMetrics>
 #include <QSizePolicy>
@@ -35,200 +36,171 @@
 
 #include <Plasma/Theme>
 #include <Plasma/PushButton>
+#include <Plasma/Label>
+#include <Plasma/Frame>
 
 CalculatorApplet::CalculatorApplet( QObject *parent, const QVariantList &args )
-    : Plasma::Applet( parent, args )
+      : Plasma::PopupApplet( parent, args ),
+      m_widget(0),
+      m_layout(0)
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
-    m_layout = 0;
     resize(300,300);
     setMinimumSize(50,50);
+    setPopupIcon("accessories-calculator");
 }
 
-void CalculatorApplet::init()
+QGraphicsWidget *CalculatorApplet::graphicsWidget()
 {
-    m_proxy = new QGraphicsProxyWidget(this);
-    m_layout = new QGraphicsGridLayout;
+    if (!m_widget) {
+        m_widget = new QGraphicsWidget(this);
+        m_layout = new QGraphicsGridLayout(m_widget);
 
-    previousAddSubOperation=calcNone;
-    previousMulDivOperation=calcNone;
+        previousAddSubOperation=calcNone;
+        previousMulDivOperation=calcNone;
 
-    int buttonX,buttonY;
+        int buttonX,buttonY;
 
-    inputText = QString('0');
-    sum = 0;
-    factor = 0;
-    waitingForDigit = true;
+        inputText = QString('0');
+        sum = 0;
+        factor = 0;
+        waitingForDigit = true;
 
-    mOutputDisplay = new QLabel;
-    mOutputDisplay->setWordWrap( true );
-    QGraphicsProxyWidget *w = new QGraphicsProxyWidget;
-    w->setWidget( mOutputDisplay );
-    m_layout->addItem( w, 0, 0,1,5 );
-    mOutputDisplay->setAlignment(Qt::AlignRight);
-    QFont font = Plasma::Theme::defaultTheme()->font( Plasma::Theme::DefaultFont );
-    font.setBold(true);
-    font.setPointSize(16);
-    QFontMetrics metric(font);
-    mOutputDisplay->setMinimumSize(50,metric.height());
-    mOutputDisplay->setFont(font);
-    mOutputDisplay->setText(inputText);
-    mOutputDisplay->setVisible(true);
+        Plasma::Frame *displayFrame = new Plasma::Frame(this);
+        displayFrame->setFrameShadow(Plasma::Frame::Sunken);
+        QGraphicsLinearLayout *frameLayout = new QGraphicsLinearLayout(displayFrame);
+        mOutputDisplay = new Plasma::Label;
+        mOutputDisplay->nativeWidget()->setWordWrap( true );
+        frameLayout->addItem(mOutputDisplay);
+        m_layout->addItem( displayFrame, 0, 0, 1, 4 );
+        mOutputDisplay->nativeWidget()->setAlignment(Qt::AlignRight);
+        QFont font = Plasma::Theme::defaultTheme()->font( Plasma::Theme::DefaultFont );
+        font.setBold(true);
+        font.setPointSize(16);
+        QFontMetrics metric(font);
+        mOutputDisplay->setMinimumSize(50,metric.height());
+        mOutputDisplay->nativeWidget()->setFont(font);
+        mOutputDisplay->nativeWidget()->setText(inputText);
 
-    mButtonDigit[0] = new Plasma::PushButton( this );
-    QFontMetrics metric1(mButtonDigit[0]->font());
-    buttonY=metric1.height()*1.3;
-    buttonX=metric1.width("00");
-    mButtonDigit[0]->setText( QString::number(0) );
-    connect( mButtonDigit[0], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
-    mButtonDigit[0]->setMinimumSize(buttonX,buttonY);
-    mButtonDigit[0]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-    mButtonDigit[0]->setVisible( true );
-    m_layout->addItem( mButtonDigit[0], 5,1,1,2 );
+        mButtonDigit[0] = new Plasma::PushButton(m_widget);
+        QFontMetrics metric1(mButtonDigit[0]->font());
+        buttonY=metric1.height()*1.3;
+        buttonX=metric1.width("00");
+        mButtonDigit[0]->setText( QString::number(0) );
+        connect( mButtonDigit[0], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
+        mButtonDigit[0]->setMinimumSize(buttonX,buttonY);
+        mButtonDigit[0]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+        mButtonDigit[0]->setVisible( true );
+        m_layout->addItem( mButtonDigit[0], 5,0,1,2 );
 
+        for (int i = 1; i < NumDigitButtons; i++) {
+            int row = ((9 - i) / 3) + 2;
+            int column = ((i - 1) % 3);
+            mButtonDigit[i] = new Plasma::PushButton(m_widget);
+            mButtonDigit[i]->setText( QString::number(i) );
+            connect( mButtonDigit[i], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
+            mButtonDigit[i]->setVisible( true );
+            mButtonDigit[i]->setMinimumSize(buttonX,buttonY);
+            mButtonDigit[i]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+            m_layout->addItem( mButtonDigit[i], row, column );
+        }
 
-    for (int i = 1; i < NumDigitButtons; i++) {
-        int row = ((9 - i) / 3) + 2;
-        int column = ((i - 1) % 3) + 1;
-        mButtonDigit[i] = new Plasma::PushButton( this );
-        mButtonDigit[i]->setText( QString::number(i) );
-        connect( mButtonDigit[i], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
-        mButtonDigit[i]->setVisible( true );
-	mButtonDigit[i]->setMinimumSize(buttonX,buttonY);
-	mButtonDigit[i]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-        m_layout->addItem( mButtonDigit[i], row,column );
+        mButtonDecimal = new Plasma::PushButton(m_widget);
+        mButtonDecimal->setText( KGlobal::locale()->decimalSymbol() );
+        m_layout->addItem( mButtonDecimal, 5, 2 );
+        connect( mButtonDecimal, SIGNAL( clicked() ), this, SLOT( slotDecimalClicked() ) );
+        mButtonDecimal->setVisible( true );
+        mButtonDecimal->setMinimumSize(buttonX,buttonY);
+        mButtonDecimal->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+
+        mButtonEquals = new Plasma::PushButton(m_widget);
+        mButtonEquals->setText( i18nc("The = button of the calculator", "=") );
+        mButtonEquals->setMinimumSize(buttonX,buttonY);
+        mButtonEquals->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+        m_layout->addItem( mButtonEquals, 4, 3, 2,1 );
+
+        connect( mButtonEquals, SIGNAL( clicked() ), this, SLOT( slotEqualsClicked() ) );
+        mButtonEquals->setVisible( true );
+
+        mButtonAdd = new Plasma::PushButton(m_widget);
+        mButtonAdd->setText( i18nc("The + button of the calculator", "+") );
+        mButtonAdd->setMinimumSize(buttonX,buttonY);
+        mButtonAdd->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+
+        m_layout->addItem( mButtonAdd, 3, 3 );
+        connect( mButtonAdd, SIGNAL( clicked() ), this, SLOT( slotAddClicked() ) );
+        mButtonAdd->setVisible( true );
+
+        mButtonSubtract = new Plasma::PushButton(m_widget);
+        mButtonSubtract->setText( i18nc("The − button of the calculator", "−") );
+        mButtonSubtract->setMinimumSize(buttonX,buttonY);
+        mButtonSubtract->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+        m_layout->addItem( mButtonSubtract, 2, 3 );
+
+        connect( mButtonSubtract, SIGNAL( clicked() ), this, SLOT( slotSubtractClicked() ) );
+        mButtonSubtract->setVisible( true );
+
+        mButtonMultiply = new Plasma::PushButton(m_widget);
+        mButtonMultiply->setText( i18nc("The × button of the calculator", "×") );
+        mButtonMultiply->setMinimumSize(buttonX,buttonY);
+        mButtonMultiply->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+        m_layout->addItem( mButtonMultiply, 1, 2 );
+
+        connect( mButtonMultiply, SIGNAL( clicked() ), this, SLOT( slotMultiplyClicked() ) );
+        mButtonMultiply->setVisible( true );
+
+        mButtonDivide = new Plasma::PushButton(m_widget);
+        mButtonDivide->setText( i18nc("The ∕ button of the calculator", "∕") );
+        mButtonDivide->setMinimumSize(buttonX,buttonY);
+        mButtonDivide->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+
+        m_layout->addItem( mButtonDivide, 1, 1 );
+
+        connect( mButtonDivide, SIGNAL( clicked() ), this, SLOT( slotDivideClicked() ) );
+        mButtonDivide->setVisible( true );
+
+        mButtonClear = new Plasma::PushButton(m_widget);
+        mButtonClear->setText( i18nc("The C button of the calculator", "C") );
+        mButtonClear->setMinimumSize(buttonX,buttonY);
+        mButtonClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+
+        m_layout->addItem( mButtonClear, 1, 0 );
+        connect( mButtonClear, SIGNAL( clicked() ), this, SLOT( slotClearClicked() ) );
+        mButtonClear->setVisible( true );
+
+        mButtonAllClear = new Plasma::PushButton(m_widget);
+        mButtonAllClear->setText( i18nc("The AC button of the calculator", "AC") );
+        mButtonAllClear->setMinimumSize(buttonX,buttonY);
+        mButtonAllClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
+
+        m_layout->addItem( mButtonAllClear, 1, 3);
+        connect( mButtonAllClear, SIGNAL( clicked() ), this, SLOT( slotAllClearClicked() ) );
+        mButtonAllClear->setVisible( true );
+
+        // the following three lines are a complete pain and only partially work.
+        // this should be moved into convenience methods in Applet, or TT needs to improve
+        // how this works in QGraphicsWidget.
+        qreal left, top, right, bottom;
+        getContentsMargins(&left, &top, &right, &bottom);
+        m_widget->setMinimumSize(m_layout->sizeHint(Qt::MinimumSize) + QSizeF(left + right, top + bottom));
+        m_widget->resize(300,300);
+
+        QAction *copy = new QAction(i18n( "Copy" ), this);
+        actions.append(copy);
+        connect(copy, SIGNAL(triggered(bool)), this, SLOT(slotCopy()));
+
+        QAction *paste = new QAction(i18n( "Paste" ), this);
+        actions.append(paste);
+        connect(paste, SIGNAL(triggered(bool)), this, SLOT(slotPaste()));
     }
 
-
-    mButtonDecimal = new Plasma::PushButton( this );
-    mButtonDecimal->setText( KGlobal::locale()->decimalSymbol() );
-    m_layout->addItem( mButtonDecimal, 5, 3 );
-    connect( mButtonDecimal, SIGNAL( clicked() ), this, SLOT( slotDecimalClicked() ) );
-    mButtonDecimal->setVisible( true );
-    mButtonDecimal->setMinimumSize(buttonX,buttonY);
-    mButtonDecimal->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-
-    mButtonEquals = new Plasma::PushButton( this );
-    mButtonEquals->setText( "=" );
-    mButtonEquals->setMinimumSize(buttonX,buttonY);
-    mButtonEquals->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-    m_layout->addItem( mButtonEquals, 4, 4,2,1 );
-
-    connect( mButtonEquals, SIGNAL( clicked() ), this, SLOT( slotEqualsClicked() ) );
-    mButtonEquals->setVisible( true );
-
-    mButtonAdd = new Plasma::PushButton( this );
-    mButtonAdd->setText( "+" );
-    mButtonAdd->setMinimumSize(buttonX,buttonY);
-    mButtonAdd->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-
-    m_layout->addItem( mButtonAdd, 3, 4 );
-    connect( mButtonAdd, SIGNAL( clicked() ), this, SLOT( slotAddClicked() ) );
-    mButtonAdd->setVisible( true );
-
-    mButtonSubtract = new Plasma::PushButton( this );
-    mButtonSubtract->setText( "-" );
-    mButtonSubtract->setMinimumSize(buttonX,buttonY);
-    mButtonSubtract->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-    m_layout->addItem( mButtonSubtract, 2, 4 );
-
-    connect( mButtonSubtract, SIGNAL( clicked() ), this, SLOT( slotSubtractClicked() ) );
-    mButtonSubtract->setVisible( true );
-
-    mButtonMultiply = new Plasma::PushButton( this );
-    mButtonMultiply->setText( "X" );
-    mButtonMultiply->setMinimumSize(buttonX,buttonY);
-    mButtonMultiply->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-    m_layout->addItem( mButtonMultiply, 1, 3 );
-
-    connect( mButtonMultiply, SIGNAL( clicked() ), this, SLOT( slotMultiplyClicked() ) );
-    mButtonMultiply->setVisible( true );
-
-    mButtonDivide = new Plasma::PushButton( this );
-    mButtonDivide->setText( "/" );
-    mButtonDivide->setMinimumSize(buttonX,buttonY);
-    mButtonDivide->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-
-    m_layout->addItem( mButtonDivide, 1, 2 );
-
-    connect( mButtonDivide, SIGNAL( clicked() ), this, SLOT( slotDivideClicked() ) );
-    mButtonDivide->setVisible( true );
-
-    mButtonClear = new Plasma::PushButton( this );
-    mButtonClear->setText( "C" );
-    mButtonClear->setMinimumSize(buttonX,buttonY);
-    mButtonClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-
-    m_layout->addItem( mButtonClear, 1, 1 );
-    connect( mButtonClear, SIGNAL( clicked() ), this, SLOT( slotClearClicked() ) );
-    mButtonClear->setVisible( true );
-
-    mButtonAllClear = new Plasma::PushButton( this );
-    mButtonAllClear->setText( "AC" );
-    mButtonAllClear->setMinimumSize(buttonX,buttonY);
-    mButtonAllClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
-
-    m_layout->addItem( mButtonAllClear, 1, 4);
-    connect( mButtonAllClear, SIGNAL( clicked() ), this, SLOT( slotAllClearClicked() ) );
-    mButtonAllClear->setVisible( true );
-
-    m_proxy->show();
-    setLayout(m_layout);
-
-    // the following three lines are a complete pain and only partially work.
-    // this should be moved into convenience methods in Applet, or TT needs to improve
-    // how this works in QGraphicsWidget.
-    qreal left, top, right, bottom;
-    getContentsMargins(&left, &top, &right, &bottom);
-    setMinimumSize(m_layout->sizeHint(Qt::MinimumSize) + QSizeF(left + right, top + bottom));
-
-    QAction *copy = new QAction(i18n( "Copy" ), this);
-    actions.append(copy);
-    connect(copy, SIGNAL(triggered(bool)), this, SLOT(slotCopy()));
-
-    QAction *paste = new QAction(i18n( "Paste" ), this);
-    actions.append(paste);
-    connect(paste, SIGNAL(triggered(bool)), this, SLOT(slotPaste()));
+    return m_widget;
 }
 
 CalculatorApplet::~CalculatorApplet()
 {
 }
 
-void CalculatorApplet::constraintsEvent(Plasma::Constraints constraints)
-{
-    if (constraints & Plasma::SizeConstraint) {
-        if (layout() && (layout()->minimumSize().width() > size().width() ||
-               layout()->minimumSize().height() > size().height())) {
-            for (int i = 0; i < 10; ++i) {
-                mButtonDigit[i]->setVisible( false );
-            }
-
-            mButtonDecimal->setVisible( false );
-            mButtonAdd->setVisible( false );
-            mButtonSubtract->setVisible( false );
-            mButtonMultiply->setVisible( false );
-            mButtonDivide->setVisible( false );
-            mButtonEquals->setVisible( false );
-            mButtonClear->setVisible( false );
-            mButtonAllClear->setVisible( false );
-        } else {
-            for (int i = 0; i < 10; ++i) {
-                mButtonDigit[i]->setVisible( true );
-            }
-
-            mButtonDecimal->setVisible( true );
-            mButtonAdd->setVisible( true );
-            mButtonSubtract->setVisible( true );
-            mButtonMultiply->setVisible( true );
-            mButtonDivide->setVisible( true );
-            mButtonEquals->setVisible( true );
-            mButtonClear->setVisible( true );
-            mButtonAllClear->setVisible( true );
-        }
-    }
-}
-
 void CalculatorApplet::keyPressEvent ( QKeyEvent * event )
 {
     switch( event->key() )
@@ -253,6 +225,7 @@ void CalculatorApplet::keyPressEvent ( QKeyEvent * event )
         event->accept();
         break;
     }
+    case Qt::Key_Asterisk:
     case Qt::Key_multiply:
     {
         mButtonMultiply->nativeWidget()->animateClick();
@@ -326,6 +299,7 @@ void CalculatorApplet::keyPressEvent ( QKeyEvent * event )
         break;
     }
     case Qt::Key_Comma:
+    case Qt::Key_Period:
     {
         mButtonDecimal->nativeWidget()->animateClick();
         event->accept();
@@ -360,7 +334,7 @@ void CalculatorApplet::slotDigitClicked()
 
     inputText += QString::number(newDigit);
 
-    if (!inputText.contains('.')) {
+    if (!inputText.contains(KGlobal::locale()->decimalSymbol())) {
       //If there is no decimal, then we need to reformat the number
       double currentValue = inputText.toDouble();
       QString localizedString = KGlobal::locale()->formatNumber(currentValue, 0);
@@ -379,12 +353,12 @@ void CalculatorApplet::slotDecimalClicked()
 
     if (waitingForDigit) {
         inputText = '0';
-	mOutputDisplay->setText(inputText);
+        mOutputDisplay->setText(inputText);
     }
-    //TODO use locale comma
-    if (!inputText.contains('.')) {
-        inputText += '.';
-	mOutputDisplay->setText(mOutputDisplay->text()+'.');
+
+    if (!inputText.contains(KGlobal::locale()->decimalSymbol())) {
+        inputText += KGlobal::locale()->decimalSymbol();
+        mOutputDisplay->setText(mOutputDisplay->text()+KGlobal::locale()->decimalSymbol());
     }
 
     waitingForDigit = false;
@@ -394,7 +368,7 @@ void CalculatorApplet::slotAddClicked()
 {
     double currentValue = inputText.toDouble();
 
-    //We need to prevent the rounding that is occuring here!
+    //We need to prevent the rounding that is occurring here!
 
     //kDebug() << "'+' was clixed, parsed to: " << currentValue << endl;
 
@@ -479,26 +453,27 @@ void CalculatorApplet::slotEqualsClicked()
     }
 
     //We use the 'g' formatted to figure out whether this if an integer
-    inputText = QString::number(sum, 'f', 6);
+    inputText = KGlobal::locale()->formatNumber(sum, 6);
 
-    QString leftSide  = inputText.left(inputText.indexOf('.'));
-    if (leftSide.size()>3)
-      leftSide = KGlobal::locale()->formatNumber(leftSide.toDouble(), 0);
+    int decimalIndex = inputText.indexOf(KGlobal::locale()->decimalSymbol());
 
-    QString rightSide = inputText.right(inputText.size()-inputText.indexOf('.'));
-    while (rightSide.size() > 1 && rightSide.endsWith('0')) {
-      rightSide = rightSide.left(rightSide.size()-1);
+    if (decimalIndex != -1) {
+        while (inputText.size() > decimalIndex && inputText.endsWith('0')) {
+            inputText = inputText.left(inputText.size() - 1);
+        }
+
+        if (inputText.endsWith(KGlobal::locale()->decimalSymbol())) {
+            inputText = inputText.left(inputText.size() - 1);
+        }
     }
-    if (rightSide.endsWith('.'))
-      rightSide.clear();
-    mOutputDisplay->setText(leftSide+rightSide);
+
+    mOutputDisplay->setText(inputText);
 
     sum = 0.0;
     factor = 0;
-    previousAddSubOperation=calcNone;
-    previousMulDivOperation=calcNone;
+    previousAddSubOperation = calcNone;
+    previousMulDivOperation = calcNone;
     waitingForDigit = true;
-
 }
 
 void CalculatorApplet::slotClearClicked()
@@ -537,7 +512,7 @@ bool CalculatorApplet::calculate(double newValue, calcOperator oldOperator)
         if (newValue != 0.0)
             factor /= newValue;
         else {
-            handleError(i18n("ERROR: DIV BY O"));
+            handleError(i18n("ERROR: DIV BY 0"));
             return false;
         }
         break;
