@@ -48,48 +48,137 @@
 #include <Plasma/Svg>
 #include <Plasma/Theme>
 #include <Plasma/DataEngine>
-#include <Plasma/Flash>
-#include <Plasma/Icon>
+#include <Plasma/Service>
+#include <Plasma/FlashingLabel>
+#include <Plasma/IconWidget>
+#include <Plasma/SvgWidget>
 #include <Plasma/TextEdit>
+#include <Plasma/Frame>
+#include <Plasma/ServiceJob>
 
 Q_DECLARE_METATYPE(Plasma::DataEngine::Data)
 
 Twitter::Twitter(QObject *parent, const QVariantList &args)
-    : Plasma::Applet(parent, args),
-      m_lastTweet(0), m_wallet(0), m_walletWait(None),
+    : Plasma::PopupApplet(parent, args),
+      m_graphicsWidget(0),
+      m_newTweets(0),
+      m_service(0),
+      m_profileService(0),
+      m_lastTweet(0),
+      m_wallet(0),
+      m_walletWait(None),
       m_colorScheme(0)
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     setHasConfigurationInterface(true);
-    resize(200, 350);
+    setPopupIcon("view-pim-journal");
 }
 
 void Twitter::init()
 {
-    m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());
+    m_flash = new Plasma::FlashingLabel(this);
     m_theme = new Plasma::Svg(this);
     m_theme->setImagePath("widgets/twitter");
+    m_theme->setContainsMultipleImages(true);
+}
+
+void Twitter::constraintsEvent(Plasma::Constraints constraints)
+{
+    //i am an icon?
+    if ((constraints|Plasma::SizeConstraint || constraints|Plasma::FormFactorConstraint) &&
+        layout()->itemAt(0) != m_graphicsWidget) {
+        paintIcon();
+    }
+}
+
+void Twitter::paintIcon()
+{
+    int size = qMin(contentsRect().width(), contentsRect().height());
+    if (size < 1) {
+        size = KIconLoader::SizeSmall;
+    }
+
+    QPixmap icon(size, size);
+    if (m_popupIcon.isNull()) {
+        icon = KIconLoader::global()->loadIcon("view-pim-journal", KIconLoader::NoGroup, size);
+    } else {
+        icon.fill(Qt::transparent);
+    }
+
+    QPainter p(&icon);
+    p.drawPixmap(icon.rect(), m_popupIcon, m_popupIcon.rect());
+    //4.3: a notification system for popupapplets would be cool
+    if (m_newTweets > 0) {
+        QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
+        QFontMetrics fm(font);
+        QRect textRect(fm.boundingRect(QString::number(m_newTweets)));
+        int textSize = qMax(textRect.width(), textRect.height());
+        textRect.setSize(QSize(textSize, textSize));
+        textRect.moveBottomRight(icon.rect().bottomRight());
+
+        QColor c(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
+        c.setAlphaF(0.6);
+
+        p.setBrush(c);
+        p.setPen(Qt::NoPen);
+        p.setRenderHints(QPainter::Antialiasing);
+        p.drawEllipse(textRect);
+
+        p.setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
+        p.drawText(textRect, Qt::AlignCenter, QString::number(m_newTweets));
+    }
+    p.end();
+
+    setPopupIcon(icon);
+}
+
+void Twitter::popupEvent(bool show)
+{
+    if (show) {
+        m_newTweets = 0;
+        paintIcon();
+    }
+}
+
+void Twitter::focusInEvent(QFocusEvent *event)
+{
+    Q_UNUSED(event);
+
+    m_statusEdit->setFocus();
+}
+
+QGraphicsWidget *Twitter::graphicsWidget()
+{
+    if (m_graphicsWidget) {
+        return m_graphicsWidget;
+    }
+
+    m_graphicsWidget = new QGraphicsWidget(this);
+    m_graphicsWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
+
     //config stuff
     KConfigGroup cg = config();
-    m_username = cg.readEntry( "username" );
-    m_password = KStringHandler::obscure( cg.readEntry( "password" ) );
-    m_historySize = cg.readEntry( "historySize", 2 );
-    m_historyRefresh = cg.readEntry( "historyRefresh", 5 );
-    m_includeFriends = cg.readEntry( "includeFriends", true );
+    m_username = cg.readEntry("username");
+    m_password = KStringHandler::obscure(cg.readEntry("password"));
+    m_historySize = cg.readEntry("historySize", 2);
+    m_historyRefresh = cg.readEntry("historyRefresh", 5);
+    m_includeFriends = cg.readEntry("includeFriends", true);
 
     m_engine = dataEngine("twitter");
     if (! m_engine->isValid()) {
-        setFailedToLaunch(true, "Failed to load twitter DataEngine");
-        return;
+        setFailedToLaunch(true, i18n("Failed to load twitter DataEngine"));
+        return m_graphicsWidget;
     }
 
     //ui setup
-    m_layout = new QGraphicsLinearLayout( Qt::Vertical );
-    setLayout( m_layout );
-    m_layout->setSpacing( 0 );
+    m_layout = new QGraphicsLinearLayout( Qt::Vertical, m_graphicsWidget );
+    m_layout->setSpacing( 3 );
 
     QGraphicsLinearLayout *flashLayout = new QGraphicsLinearLayout( Qt::Horizontal );
-    m_flash = new Plasma::Flash( this );
+
+    m_flash->setAutohide( true );
     m_flash->setMinimumSize( 0, 20 );
     m_flash->setColor( Qt::gray );
     QFont fnt = qApp->font();
@@ -97,28 +186,43 @@ void Twitter::init()
     QFontMetrics fm( fnt );
     m_flash->setFont( fnt );
     m_flash->flash( "", 20000 );
-    flashLayout->addItem( m_flash );
-    flashLayout->setItemSpacing( 0, 175 );
+    m_flash->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+
+    QGraphicsLinearLayout *titleLayout = new QGraphicsLinearLayout( Qt::Vertical );
+    Plasma::SvgWidget *svgTitle = new Plasma::SvgWidget(m_theme, "twitter", this);
+    svgTitle->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    svgTitle->setPreferredSize(75, 14);
+    titleLayout->addItem(svgTitle);
+
+    flashLayout->addItem(m_flash);
+    flashLayout->setStretchFactor(m_flash, 2);
+    flashLayout->addItem( titleLayout );
     m_layout->addItem( flashLayout );
 
-    m_headerLayout = new QGraphicsLinearLayout( Qt::Horizontal );
+    Plasma::Frame *headerFrame = new Plasma::Frame(this);
+    m_headerLayout = new QGraphicsLinearLayout( Qt::Horizontal, headerFrame );
     m_headerLayout->setContentsMargins( 5, 5, 5, 10 );
     m_headerLayout->setSpacing( 5 );
-    m_layout->addItem( m_headerLayout );
+    m_layout->addItem( headerFrame );
 
 
-    m_icon = new Plasma::Icon( this );
-    m_icon->setIcon( KIcon( "user-identity" ) );
-    m_icon->setText( m_username );
+    m_icon = new Plasma::IconWidget(this);
+    m_icon->setIcon(KIcon("user-identity"));
+    m_icon->setText(m_username);
     QSizeF iconSize = m_icon->sizeFromIconSize(48);
     m_icon->setMinimumSize( iconSize );
     m_icon->setMaximumSize( iconSize );
     m_headerLayout->addItem( m_icon );
 
-    m_statusEdit = new Plasma::TextEdit();
+    Plasma::Frame *statusEditFrame = new Plasma::Frame(this);
+    statusEditFrame->setFrameShadow(Plasma::Frame::Sunken);
+    QGraphicsLinearLayout *statusEditLayout = new QGraphicsLinearLayout(statusEditFrame);
+    m_statusEdit = new Plasma::TextEdit(this);
     m_statusEdit->nativeWidget()->setFrameShape( QFrame::NoFrame );
     m_statusEdit->nativeWidget()->setTextBackgroundColor( QColor(0,0,0,0) );
     m_statusEdit->nativeWidget()->viewport()->setAutoFillBackground( false );
+    connect(m_statusEdit, SIGNAL(textChanged()), this, SLOT(editTextChanged()));
+    statusEditLayout->addItem(m_statusEdit);
 
     //FIXME: m_statusEdit->setTextColor( m_colorScheme->foreground().color() );
     // seems to have no effect
@@ -126,58 +230,64 @@ void Twitter::init()
     editPal.setColor(QPalette::Text, m_colorScheme->foreground().color());
     m_statusEdit->nativeWidget()->setPalette(editPal);
     m_statusEdit->nativeWidget()->installEventFilter(this);
-    m_headerLayout->addItem( m_statusEdit );
+    m_headerLayout->addItem( statusEditFrame );
 
+    m_layout->addStretch();
     m_layout->addStretch();
 
     //hook up some sources
-    m_engine->connectSource("LatestImage", this);
+    m_engine->connectSource("UserImages", this);
     m_engine->connectSource("Error:UserImages", this);
     m_engine->connectSource("Error", this);
 
     //set things in motion
-    if(! m_username.isEmpty()) {
-        if (m_password.isEmpty()) {
-            m_walletWait = Read;
-            getWallet();
-        } else { //use config value
-            setAuth();
-        }
-    }else{
+    if (m_username.isEmpty()) {
         setAuthRequired(true);
+    } else if (m_password.isEmpty()) {
+        kDebug() << "started, password is not in config file, trying wallet";
+        m_walletWait = Read;
+        getWallet();
+    } else { //use config value
+        kDebug() << "password was in config file, using that to get twitter history";
+        downloadHistory();
     }
+
+    return m_graphicsWidget;
 }
 
 void Twitter::getWallet()
 {
     //TODO: maybe Plasma in general should handle the wallet
-    if (m_wallet) {
-        //user must be a dumbass. kill that old attempt.
-        delete m_wallet;
-    }
+    delete m_wallet;
+
     QGraphicsView *v = view();
-    WId w;
-    if (!v) {
-        kDebug() << "eek! no view!";
-        w=0;
-    } else {
-        w=v->winId();
+    WId w = 0;
+    if (v) {
+        w = v->winId();
     }
-    m_wallet = KWallet::Wallet::openWallet( KWallet::Wallet::NetworkWallet(),
-            w, KWallet::Wallet::Asynchronous );
+
+    kDebug() << "opening wallet";
+    m_wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(),
+                                           w, KWallet::Wallet::Asynchronous);
+
     if (m_walletWait == Write) {
-        connect( m_wallet, SIGNAL( walletOpened(bool) ), SLOT( writeWallet(bool) ) );
+        connect(m_wallet, SIGNAL(walletOpened(bool)), SLOT(writeWallet(bool)));
     } else {
-        connect( m_wallet, SIGNAL( walletOpened(bool) ), SLOT( readWallet(bool) ) );
+        connect(m_wallet, SIGNAL(walletOpened(bool)), SLOT(readWallet(bool)));
     }
 }
 
 void Twitter::writeWallet(bool success)
 {
-    //kDebug();
-    if (!(success && enterWalletFolder(QString::fromLatin1("Plasma-Twitter"))
-                && (m_wallet->writePassword(m_username, m_password) == 0))) {
-        kDebug() << "failed to write password";
+    kDebug() << success;
+    if (success &&
+        enterWalletFolder(QString::fromLatin1("Plasma-Twitter")) &&
+        (m_wallet->writePassword(m_username, m_password) == 0)) {
+        kDebug() << "successfully put password in wallet, removing from config file";
+        config().deleteEntry("password");
+        emit configNeedsSaving();
+    } else {
+        kDebug() << "failed to store password in wallet, putting into config file instead";
         writeConfigPassword();
     }
     m_walletWait = None;
@@ -187,15 +297,21 @@ void Twitter::writeWallet(bool success)
 
 void Twitter::readWallet(bool success)
 {
-    //kDebug();
+    kDebug() << success;
     QString pwd;
-    if (success && enterWalletFolder(QString::fromLatin1("Plasma-Twitter")) 
-            && (m_wallet->readPassword(m_username, pwd) == 0)) {
+    if (success &&
+        enterWalletFolder(QString::fromLatin1("Plasma-Twitter")) &&
+        (m_wallet->readPassword(m_username, pwd) == 0)) {
+        kDebug() << "successfully retreived password from wallet";
         m_password = pwd;
-        setAuth();
-    } else {
+        downloadHistory();
+    } else if (m_password.isEmpty()) {
+        //FIXME: when out of string freeze, tell the user WHY they need
+        //       to configure the widget;
+        setConfigurationRequired(true);
         kDebug() << "failed to read password";
     }
+
     m_walletWait = None;
     delete m_wallet;
     m_wallet = 0;
@@ -210,16 +326,9 @@ bool Twitter::enterWalletFolder(const QString &folder)
         kDebug() << "failed to open folder" << folder;
         return false;
     }
-    return true;
-}
 
-void Twitter::setAuth()
-{
-    Plasma::DataEngine::Data conf = m_engine->property("config").value<Plasma::DataEngine::Data>();
-    conf[m_username] = m_password;
-    m_engine->setProperty("config", QVariant::fromValue(conf));
-//     m_statusEdit->setVisible( !( m_username.isEmpty() || m_password.isEmpty() ) );
-    downloadHistory();
+    kDebug() << "wallet now on folder" << folder;
+    return true;
 }
 
 void Twitter::setAuthRequired(bool required)
@@ -230,17 +339,17 @@ void Twitter::setAuthRequired(bool required)
 
 void Twitter::writeConfigPassword()
 {
-    kDebug();
+    //kDebug();
+    //TODO: don't use "Yes" and "No", but replace with meaningful labels!
     if (KMessageBox::warningYesNo(0, i18n("Failed to access kwallet. Store password in config file instead?"))
             == KMessageBox::Yes) {
-        KConfigGroup cg = config();
-        cg.writeEntry( "password", KStringHandler::obscure(m_password) );
+        config().writeEntry("password", KStringHandler::obscure(m_password));
     }
 }
 
 void Twitter::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
 {
-    kDebug() << source;
+    //kDebug() << source;
     if (data.isEmpty()) {
         if (source.startsWith("Error")) {
             m_flash->kill(); //FIXME only clear it if it was showing an error msg
@@ -257,47 +366,47 @@ void Twitter::dataUpdated(const QString& source, const Plasma::DataEngine::Data
         int newCount = 0;
         uint maxId = m_lastTweet;
         foreach (const QString &id, data.keys()) {
-            uint i=id.toUInt();
+            uint i = id.toUInt();
             if (i > m_lastTweet) {
                 newCount++;
                 QVariant v = data.value(id);
                 //Warning: This function is not available with MSVC 6
                 Plasma::DataEngine::Data t = v.value<Plasma::DataEngine::Data>();
-                m_tweetMap[id] = t;
+                m_tweetMap[i] = t;
                 if (i > maxId) {
                     maxId = i;
                 }
             }
         }
         m_lastTweet = maxId;
-        m_flash->flash( i18np( "1 new tweet", "%1 new tweets", qMin(newCount, m_historySize) ), 20*1000 );
+        m_newTweets = qMin(newCount, m_historySize);
+        m_flash->flash( i18np( "1 new tweet", "%1 new tweets", m_newTweets ), 20*1000 );
         showTweets();
-    } else if (source == "LatestImage") {
-        QString user = data.begin().key();
-        if (user.isEmpty()) {
-            return;
-        }
-        QPixmap pm = data[user].value<QPixmap>();
-        if( !pm.isNull() ) {
-            if( user == m_username ) {
-                QAction *profile = new QAction(QIcon(pm), m_username, this);
-                profile->setData(m_username);
-                
-                QSizeF iconSize = m_icon->sizeFromIconSize(48);
-                m_icon->setAction(profile);
-                m_icon->setMinimumSize( iconSize );
-                m_icon->setMaximumSize( iconSize );
-                connect(profile, SIGNAL(triggered(bool)), this, SLOT(openProfile()));
+    } else if (source == "UserImages") {
+        foreach (const QString &user, data.keys()) {
+            QPixmap pm = data[user].value<QPixmap>();
+
+            if( !pm.isNull() ) {
+                if( user == m_username ) {
+                    QAction *profile = new QAction(QIcon(pm), m_username, this);
+                    profile->setData(m_username);
+
+                    QSizeF iconSize = m_icon->sizeFromIconSize(48);
+                    m_icon->setAction(profile);
+                    m_icon->setMinimumSize( iconSize );
+                    m_icon->setMaximumSize( iconSize );
+                    connect(profile, SIGNAL(triggered(bool)), this, SLOT(openProfile()));
+                }
+                m_pictureMap[user] = pm;
+                //TODO it would be nice to check whether the updated image is actually in use
+                showTweets();
             }
-            m_pictureMap[user] = pm;
-            //TODO it would be nice to check whether the updated image is actually in use
-            showTweets();
         }
     } else if (source.startsWith("Error")) {
         QString desc = data["description"].toString();
 
         if (desc == "Authentication required"){
-            setAuthRequired(true);            
+            setAuthRequired(true);
         }
 
         m_flash->flash(desc, 60 * 1000); //I'd really prefer it to stay there. and be red.
@@ -305,19 +414,28 @@ void Twitter::dataUpdated(const QString& source, const Plasma::DataEngine::Data
     updateGeometry();
 }
 
+void Twitter::themeChanged()
+{
+    delete m_colorScheme;
+    m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());
+    showTweets();
+}
+
 void Twitter::showTweets()
 {
     prepareGeometryChange();
     // Adjust the number of the TweetWidgets if the configuration has changed
     // Add more tweetWidgets if there are not enough
-    while( m_tweetWidgets.size() < m_historySize ) {
-        QGraphicsLinearLayout *tweetLayout = new QGraphicsLinearLayout( Qt::Horizontal );
+    while (m_tweetWidgets.size() < m_historySize) {
+        Plasma::Frame *tweetFrame = new Plasma::Frame(this);
+
+        QGraphicsLinearLayout *tweetLayout = new QGraphicsLinearLayout( Qt::Horizontal, tweetFrame );
         tweetLayout->setContentsMargins( 0, 5, 0, 5 );
         tweetLayout->setSpacing( 5 );
-        m_layout->insertItem( m_layout->count()-1, tweetLayout );
+        m_layout->insertItem( m_layout->count()-1, tweetFrame );
 
         KTextBrowser *c = new KTextBrowser();
-        QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget( this );
+        QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget(tweetFrame);
         proxy->setWidget( c );
         c->setCursor( Qt::ArrowCursor );
         c->setFrameShape(QFrame::NoFrame);
@@ -328,23 +446,23 @@ void Twitter::showTweets()
         c->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
         c->setReadOnly(true);
 
-        Plasma::Icon *icon = new Plasma::Icon( this );
+        Plasma::IconWidget *icon = new Plasma::IconWidget(tweetFrame);
         QSizeF iconSize = icon->sizeFromIconSize(30);
         icon->setMinimumSize( iconSize );
         icon->setMaximumSize( iconSize );
 
-        Plasma::Icon *favIcon = new Plasma::Icon( this );
+        Plasma::IconWidget *favIcon = new Plasma::IconWidget(tweetFrame);
         QSizeF favIconSize = icon->sizeFromIconSize(16);
         favIcon->setMinimumSize( favIconSize );
         favIcon->setMaximumSize( favIconSize );
 
-        tweetLayout->addItem( icon );
-        tweetLayout->addItem( proxy );
-        tweetLayout->addItem( favIcon );
+        tweetLayout->addItem(icon);
+        tweetLayout->addItem(proxy);
+        tweetLayout->addItem(favIcon);
         tweetLayout->updateGeometry();
 
         Tweet t;
-        t.layout = tweetLayout;
+        t.frame = tweetFrame;
         t.icon = icon;
         t.content = c;
         t.contentProxy = proxy;
@@ -352,26 +470,32 @@ void Twitter::showTweets()
 
         m_tweetWidgets.append( t );
     }
+
     //clear out tweet widgets if there are too many
-    while( m_tweetWidgets.size() > m_historySize ) {
-        Tweet t = m_tweetWidgets[m_tweetWidgets.size()-1];
-        m_layout->removeItem( t.layout );
-        delete t.icon;
-        delete t.content;
-        delete t.layout;
-        m_tweetWidgets.removeAt( m_tweetWidgets.size()-1 );
+    while (m_tweetWidgets.size() > m_historySize) {
+        Tweet t = m_tweetWidgets[m_tweetWidgets.size() - 1];
+        m_layout->removeItem(t.frame);
+        QAction *iconAction = t.icon->action();
+        t.icon->setAction(0);
+        delete iconAction;
+        delete t.frame;
+        m_tweetWidgets.removeAt(m_tweetWidgets.size() - 1);
     }
 
     int i = 0;
-    int pos = m_tweetMap.keys().size() - 1;
-    while(i < m_historySize && pos >= 0 ) {
-        Plasma::DataEngine::Data tweetData = m_tweetMap[m_tweetMap.keys()[pos]];
+    QMap<uint, Plasma::DataEngine::Data>::const_iterator it = m_tweetMap.constEnd();
+    while (i < m_historySize && it != m_tweetMap.constBegin()) {
+        Plasma::DataEngine::Data tweetData = *(--it);
         QString user = tweetData.value( "User" ).toString();
         QPixmap favIcon = tweetData.value("SourceFavIcon").value<QPixmap>();
 
+        if (i == 0) {
+            m_popupIcon = m_pictureMap[user];
+        }
+
         QAction *profile = new QAction(QIcon(m_pictureMap[user]), QString(), this);
         profile->setData(user);
-        
+
         Tweet t = m_tweetWidgets[i];
         t.icon->setAction(profile);
         QSizeF iconSize = t.icon->sizeFromIconSize(30);
@@ -380,12 +504,14 @@ void Twitter::showTweets()
         connect(profile, SIGNAL(triggered(bool)), this, SLOT(openProfile()));
 
         QString sourceString;
-        if( favIcon.isNull() ) {
+
+        if (favIcon.isNull()) {
             sourceString = i18n(" from %1", tweetData.value( "Source" ).toString());
         }
+
         QString html = "<table cellspacing='0' spacing='5' width='100%'>";
-        html += i18n( "<tr><td align='left' width='1%'><font color='%2'>%1</font></td><td align='right' width='auto'><p align='right'><font color='%2'>%3%4</font></p></td></tr>", user, m_colorScheme->foreground(KColorScheme::InactiveText).color().name(),
-                timeDescription( tweetData.value( "Date" ).toDateTime() ), sourceString);
+        html += QString("<tr height='1em'><td align='left' width='1%'><font color='%2'>%1</font></td><td align='right' width='auto'><p align='right'><font color='%2'>%3%4</font></p></td></tr>").arg( user).arg(m_colorScheme->foreground(KColorScheme::InactiveText).color().name())
+                .arg(timeDescription( tweetData.value( "Date" ).toDateTime() )).arg( sourceString);
         QString status = tweetData.value( "Status" ).toString();
 
         status.replace(QRegExp("((http|https)://[^\\s<>'\"]+[^!,\\.\\s<>'\"\\]])"), "<a href='\\1'>\\1</a>");
@@ -394,76 +520,67 @@ void Twitter::showTweets()
                 .arg( m_colorScheme->foreground().color().name()).arg( status );
         html += "</table>";
         t.content->setHtml( html );
-        t.content->document()->setTextWidth(t.content->width());
-        t.content->setMinimumHeight(t.content->document()->size().height());
+        t.content->document()->setDefaultStyleSheet(QString("a{color:%1} a:visited{color:%2}")
+                                            .arg( m_colorScheme->foreground(KColorScheme::LinkText).color().name())
+                                            .arg(m_colorScheme->foreground(KColorScheme::VisitedText).color().name()));
+        t.content->document()->setTextWidth(qMax(200,t.content->width()));
+        t.content->setMinimumSize(t.content->document()->size().toSize());
+        t.content->setMaximumHeight(t.content->minimumSize().height());
+
         t.content->update();
 
+        //FIXME: this hopefully would get useless in QGL of 4.5
+        qreal left, top, right, bottom;
+        t.frame->getContentsMargins(&left, &top, &right, &bottom);
+        t.frame->setMinimumHeight(qMax(t.content->size().height(), int(t.icon->geometry().bottom())) + top + bottom);
+        t.frame->setPreferredHeight(t.frame->minimumHeight());
+
         if( !favIcon.isNull() ) {
             t.favIcon->setIcon( QIcon(favIcon) );
         }
 
         ++i;
-        --pos;
     }
+
     qreal left, top, right, bottom;
     getContentsMargins(&left, &top, &right, &bottom);
-    resize(m_layout->sizeHint(Qt::PreferredSize) + QSizeF(left+right, top+bottom));
-    updateGeometry();
+    //this line break things, strange
+    m_graphicsWidget->setMinimumSize(m_layout->sizeHint(Qt::MinimumSize));
+    m_graphicsWidget->setPreferredSize(m_layout->sizeHint(Qt::PreferredSize));
+    //are we complete?
+    if (layout()->itemAt(0) == m_graphicsWidget) {
+        setMinimumSize(m_layout->sizeHint(Qt::MinimumSize) + QSizeF(left+right, top+bottom));
+        setPreferredSize(m_layout->sizeHint(Qt::PreferredSize) + QSizeF(left+right, top+bottom));
+        resize(preferredSize());
+    } else {
+        paintIcon();
+    }
 }
 
 void Twitter::createConfigurationInterface(KConfigDialog *parent)
 {
-    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
-    connect( parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()) );
-    connect( parent, SIGNAL(okClicked()), this, SLOT(configAccepted()) );
+    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
+    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
 
     QWidget *configWidget = new QWidget();
+    configUi.setupUi(configWidget);
 
-    m_usernameEdit = new KLineEdit( configWidget );
-    m_passwordEdit = new KLineEdit( configWidget );
-    m_historySizeSpinBox = new QSpinBox( configWidget );
-    m_historySizeSpinBox->setSuffix(i18n(" tweets"));
-    m_historyRefreshSpinBox = new QSpinBox( configWidget );
-    m_historyRefreshSpinBox->setSuffix(i18n(" minutes"));
-    m_checkIncludeFriends = new QCheckBox( configWidget );
-
-    QLabel *usernameLabel = new QLabel( i18n("User name:"), configWidget );
-    QLabel *passwordLabel = new QLabel( i18n("Password:"), configWidget );
-    QLabel *historyLabel = new QLabel( i18n("Timeline size:"), configWidget );
-    QLabel *historyRefreshLabel = new QLabel( i18n("Timeline refresh:"), configWidget );
-    QLabel *friendsLabel = new QLabel( i18n("Show messages of friends:"), configWidget );
-
-    m_passwordEdit->setPasswordMode( true );
-
-    QGridLayout *layout = new QGridLayout( configWidget );
-    layout->addWidget( usernameLabel, 0, 0 );
-    layout->addWidget( m_usernameEdit, 0, 1 );
-    layout->addWidget( passwordLabel, 1, 0 );
-    layout->addWidget( m_passwordEdit, 1, 1 );
-    layout->addWidget( historyLabel, 2, 0 );
-    layout->addWidget( m_historySizeSpinBox, 2, 1 );
-    layout->addWidget( historyRefreshLabel, 3, 0 );
-    layout->addWidget( m_historyRefreshSpinBox, 3, 1 );
-    layout->addWidget( friendsLabel, 4, 0);
-    layout->addWidget( m_checkIncludeFriends, 4, 1 );
-
-    m_usernameEdit->setText( m_username );
-    m_passwordEdit->setText( m_password );
-    m_historySizeSpinBox->setValue( m_historySize );
-    m_historyRefreshSpinBox->setValue( m_historyRefresh );
-    m_checkIncludeFriends->setCheckState( m_includeFriends ? Qt::Checked : Qt::Unchecked );
-
-    parent->addPage(configWidget, parent->windowTitle(), icon());
+    configUi.usernameEdit->setText(m_username);
+    configUi.passwordEdit->setText(m_password);
+    configUi.historySizeSpinBox->setValue(m_historySize);
+    configUi.historyRefreshSpinBox->setValue(m_historyRefresh);
+    configUi.checkIncludeFriends->setCheckState(m_includeFriends ? Qt::Checked : Qt::Unchecked);
 
+    parent->addPage(configWidget, i18n("General"), icon());
 }
 
 void Twitter::configAccepted()
 {
-    QString username = m_usernameEdit->text();
-    QString password = m_passwordEdit->text();
-    int historyRefresh = m_historyRefreshSpinBox->value();
-    int historySize = m_historySizeSpinBox->value();
-    bool includeFriends = (m_checkIncludeFriends->checkState() == Qt::Checked);
+    QString username = configUi.usernameEdit->text();
+    QString password = configUi.passwordEdit->text();
+    int historyRefresh = configUi.historyRefreshSpinBox->value();
+    int historySize = configUi.historySizeSpinBox->value();
+    bool includeFriends = configUi.checkIncludeFriends->isChecked();
     bool changed = false;
 
     KConfigGroup cg = config();
@@ -484,11 +601,7 @@ void Twitter::configAccepted()
     //then the user is a dumbass.
     //we're going to ignore that, which drops the read attempt
     //I hope that doesn't cause trouble.
-    //XXX if there's a value in the config, the wallet will never be read
-    //if a user saves their password in the config and later changes their mind
-    //then they might not understand that they have to delete the password from plasma-appletsrc
-    //FIXME if the username is blank, don't connect to the engine, set needsconfiguring, etc
-    if (! m_username.isEmpty() && (changed || m_password.isEmpty())) {
+    if (!m_username.isEmpty() && (changed || m_password.isEmpty())) {
         //a change in name *or* pass means we need to update the wallet
         //if the user doesn't set a password, see if it's already in our wallet
         m_walletWait = m_password.isEmpty() ? Read : Write;
@@ -518,40 +631,25 @@ void Twitter::configAccepted()
         //TODO we only *need* to wipe the map if name or includeFriends changed
         m_tweetMap.clear();
         m_lastTweet=0;
-        setAuth();
+        downloadHistory();
 
         emit configNeedsSaving();
     }
 
-    setAuthRequired(m_username.isEmpty() || m_password.isEmpty());
+    setAuthRequired(m_username.isEmpty());
 }
 
 Twitter::~Twitter()
 {
     delete m_colorScheme;
+    delete m_service;
 }
 
-void Twitter::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
+void Twitter::editTextChanged()
 {
-    Q_UNUSED(option);
-
-    m_theme->resize();
-    m_theme->paint( p, QRect(contentsRect.x()+contentsRect.width()-75, m_flash->geometry().y(), 75, 14), "twitter" );
-
-    foreach( const Tweet &t, m_tweetWidgets ) {
-        QRectF tweetRect(contentsRect.x(), t.layout->geometry().y(), contentsRect.width(), t.layout->geometry().height());
-        m_theme->paint( p, tweetRect, "tweet" );
-    }
-    QRectF headerRect(contentsRect.x(), m_headerLayout->geometry().y(), contentsRect.width(), m_headerLayout->geometry().height()-5);
-    m_theme->paint( p, headerRect, "tweet" );
-
-    p->setRenderHints( QPainter::Antialiasing | QPainter::SmoothPixmapTransform );
-    p->translate(0.5, 0.5);
-    p->setPen( m_colorScheme->shade( KColorScheme::LightShade ) );
-    p->drawRoundedRect( m_statusEdit->geometry(), 5, 5 );
+    m_flash->flash(i18np("%1 character left", "%1 characters left", 140 - m_statusEdit->nativeWidget()->toPlainText().length()), 2000);
 }
 
-
 bool Twitter::eventFilter(QObject *obj, QEvent *event)
 {
     if (obj == m_statusEdit->nativeWidget()) {
@@ -578,15 +676,22 @@ bool Twitter::eventFilter(QObject *obj, QEvent *event)
 
 void Twitter::updateStatus()
 {
-    QString status = m_username + ':' + m_statusEdit->nativeWidget()->toPlainText();
-    m_engine->setProperty( "status", status );
+    QString status = m_statusEdit->nativeWidget()->toPlainText();
+
+    delete m_service;
+    m_service = m_engine->serviceForSource(m_curTimeline);
+    KConfigGroup cg = m_service->operationDescription("update");
+    cg.writeEntry("password", m_password);
+    cg.writeEntry("status", status);
+    m_service->startOperationCall(cg);
+
     m_statusEdit->nativeWidget()->setPlainText("");
 }
 
 //what this really means now is 'reconnect to the timeline source'
 void Twitter::downloadHistory()
 {
-    kDebug() ;
+    //kDebug() ;
     if (m_username.isEmpty() || m_password.isEmpty()) {
         if (!m_curTimeline.isEmpty()) {
             m_engine->disconnectSource(m_curTimeline, this);
@@ -603,6 +708,7 @@ void Twitter::downloadHistory()
     } else {
         query = QString("Timeline:%1");
     }
+
     query = query.arg(m_username);
     if (m_curTimeline != query) {
         //ditch the old one, if needed
@@ -612,15 +718,41 @@ void Twitter::downloadHistory()
         }
         m_curTimeline = query;
     }
-    kDebug() << "Connecting to source " << query;
+
+    //kDebug() << "Connecting to source " << query << "with refresh rate" << m_historyRefresh * 60 * 1000;
     m_engine->connectSource(query, this, m_historyRefresh * 60 * 1000);
     m_engine->connectSource("Error:" + query, this);
+
+    delete m_service;
+    m_service = m_engine->serviceForSource(query);
+    KConfigGroup cg = m_service->operationDescription("auth");
+    cg.writeEntry("password", m_password);
+    m_service->startOperationCall(cg);
+    connect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
+
+    //get the profile to retrieve the user icon
+    QString profileQuery(QString("Profile:%1").arg(m_username));
+    m_engine->connectSource(profileQuery, this, m_historyRefresh * 60 * 1000);
+
+    delete m_profileService;
+    Plasma::Service *m_profileService = m_engine->serviceForSource(profileQuery);
+    KConfigGroup profileConf = m_profileService->operationDescription("auth");
+    profileConf.writeEntry("password", m_password);
+    m_profileService->startOperationCall(profileConf);
+    connect(m_profileService, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
+}
+
+void Twitter::serviceFinished(Plasma::ServiceJob *job)
+{
+    if (job->error()) {
+        m_flash->flash(job->errorString(), 2000);
+    }
 }
 
 void Twitter::openProfile()
 {
     QAction *action = qobject_cast<QAction *>(sender());
-    
+
     if (action) {
         KRun::runUrl( KUrl("http://www.twitter.com/" + action->data().toString()), "text/html", 0 );
     }
